# GUAN is an open-source python package developed and maintained by https://www.guanjihuan.com. The primary location of this package is on website https://py.guanjihuan.com.

# test

def test():
    print('\nSuccess in the installation of GUAN package!\n')